
#ifndef KASTENCONTROLLERS_EXPORT_H
#define KASTENCONTROLLERS_EXPORT_H

#ifdef KASTENCONTROLLERS_STATIC_DEFINE
#  define KASTENCONTROLLERS_EXPORT
#  define KASTENCONTROLLERS_NO_EXPORT
#else
#  ifndef KASTENCONTROLLERS_EXPORT
#    ifdef KastenControllers_EXPORTS
        /* We are building this library */
#      define KASTENCONTROLLERS_EXPORT __attribute__((visibility("default")))
#    else
        /* We are using this library */
#      define KASTENCONTROLLERS_EXPORT __attribute__((visibility("default")))
#    endif
#  endif

#  ifndef KASTENCONTROLLERS_NO_EXPORT
#    define KASTENCONTROLLERS_NO_EXPORT __attribute__((visibility("hidden")))
#  endif
#endif

#ifndef KASTENCONTROLLERS_DEPRECATED
#  define KASTENCONTROLLERS_DEPRECATED __attribute__ ((__deprecated__))
#endif

#ifndef KASTENCONTROLLERS_DEPRECATED_EXPORT
#  define KASTENCONTROLLERS_DEPRECATED_EXPORT KASTENCONTROLLERS_EXPORT KASTENCONTROLLERS_DEPRECATED
#endif

#ifndef KASTENCONTROLLERS_DEPRECATED_NO_EXPORT
#  define KASTENCONTROLLERS_DEPRECATED_NO_EXPORT KASTENCONTROLLERS_NO_EXPORT KASTENCONTROLLERS_DEPRECATED
#endif

/* NOLINTNEXTLINE(readability-avoid-unconditional-preprocessor-if) */
#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef KASTENCONTROLLERS_NO_DEPRECATED
#    define KASTENCONTROLLERS_NO_DEPRECATED
#  endif
#endif

#endif /* KASTENCONTROLLERS_EXPORT_H */
