
#ifndef OKTETAKASTENCORE_EXPORT_H
#define OKTETAKASTENCORE_EXPORT_H

#ifdef OKTETAKASTENCORE_STATIC_DEFINE
#  define OKTETAKASTENCORE_EXPORT
#  define OKTETAKASTENCORE_NO_EXPORT
#else
#  ifndef OKTETAKASTENCORE_EXPORT
#    ifdef OktetaKastenCore_EXPORTS
        /* We are building this library */
#      define OKTETAKASTENCORE_EXPORT __attribute__((visibility("default")))
#    else
        /* We are using this library */
#      define OKTETAKASTENCORE_EXPORT __attribute__((visibility("default")))
#    endif
#  endif

#  ifndef OKTETAKASTENCORE_NO_EXPORT
#    define OKTETAKASTENCORE_NO_EXPORT __attribute__((visibility("hidden")))
#  endif
#endif

#ifndef OKTETAKASTENCORE_DEPRECATED
#  define OKTETAKASTENCORE_DEPRECATED __attribute__ ((__deprecated__))
#endif

#ifndef OKTETAKASTENCORE_DEPRECATED_EXPORT
#  define OKTETAKASTENCORE_DEPRECATED_EXPORT OKTETAKASTENCORE_EXPORT OKTETAKASTENCORE_DEPRECATED
#endif

#ifndef OKTETAKASTENCORE_DEPRECATED_NO_EXPORT
#  define OKTETAKASTENCORE_DEPRECATED_NO_EXPORT OKTETAKASTENCORE_NO_EXPORT OKTETAKASTENCORE_DEPRECATED
#endif

/* NOLINTNEXTLINE(readability-avoid-unconditional-preprocessor-if) */
#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef OKTETAKASTENCORE_NO_DEPRECATED
#    define OKTETAKASTENCORE_NO_DEPRECATED
#  endif
#endif

#endif /* OKTETAKASTENCORE_EXPORT_H */
